from CupDefs import utils
from CupDefs import vars
import enum
import io
import datetime
import typing


class LogType(enum.Enum):
    DONE = utils.pdone
    ERROR = utils.perror
    WARNING = utils.pwarn
    INFO = utils.pinfo


class Logger:
    _LOGS_PATH = './last_launch.log'
    _logger_file: io.TextIOWrapper = None
    _is_logs_inited = False

    @staticmethod
    def mklog(log_type: LogType | typing.Callable, text: str):
        time = datetime.datetime.now(datetime.timezone.utc)
        time = time.strftime("%d.%m.%y-%H:%M:%S")
        match log_type:
            case utils.pdone:
                prefix = 'OK '
            case utils.perror:
                prefix = 'ERR'
            case utils.pwarn:
                prefix = 'WRN'
            case _:
                prefix = 'INF'
        prefix = '[' + prefix + '][' + time + '] '
        text = prefix + text
        if vars.PRINT_CUPDEFS_ERROR:
            log_type(text)
        if not Logger._is_logs_inited:
            if vars.PRINT_CUPDEFS_ERROR:
                utils.pwarn('Невозможно осуществить запись в файл логов - он не открыт')
            return
        Logger._logger_file.write(text + '\n')

    @staticmethod
    def mklogif(condition: bool | typing.Callable, log_type, text: str):
        if callable(condition):
            condition = condition()
        if condition:
            Logger.mklog(log_type, text)

    @staticmethod
    def init_logs(logs_path: str | None) -> bool:
        if Logger._is_logs_inited:
            return True
        try:
            if logs_path:
                Logger._LOGS_PATH = logs_path
            with open(Logger._LOGS_PATH, 'w'):  # открытие, очистка и закрытие
                pass
            Logger._logger_file = open(Logger._LOGS_PATH, 'a', encoding='utf-8')
            Logger._is_logs_inited = True
            if vars.PRINT_CUPDEFS_ERROR:
                utils.pdone('Файл логов успешно создан')
            Logger.mklog(LogType.INFO, 'Начало логирования. Время по UTC')
            return True
        except IOError:
            if vars.PRINT_CUPDEFS_ERROR:
                utils.perror('Файл логов не создан')
            return False

    @staticmethod
    def close_logs() -> bool:
        if not Logger._is_logs_inited:
            return True
        try:
            Logger.mklog(LogType.INFO, 'Конец логирования')
            Logger._logger_file.close()
            Logger._is_logs_inited = False
            if vars.PRINT_CUPDEFS_ERROR:
                utils.pdone('Файл логов успешно закрыт')
            return True
        except IOError:
            if vars.PRINT_CUPDEFS_ERROR:
                utils.pwarn('Файл логов не удалось закрыть')
            return False



import datetime
import os
import hashlib
from CupDefs import vars
from CupDefs import utils

_TODAY: None | datetime.datetime = None
_DAYS = [0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 303, 334, 365]
_FILES: None | list[str] = None


def init_vergen():
    global _TODAY, _FILES
    if _TODAY is None:
        _TODAY = datetime.datetime.now(datetime.timezone.utc)
        _FILES = []
        for root, dirs, files in os.walk(vars.VERGEN_HASH_PATH):
            for file in files:
                _FILES.append(os.path.join(root, file))


def generate() -> str | None:
    if _TODAY:
        num = 365 * (_TODAY.year - 2020) - 31 + _DAYS[(_TODAY.month - 1)] + _TODAY.day
        h = b''
        for path in _FILES:
            with open(path, 'rb') as buf:
                h += hashlib.md5(buf.read()).digest()
        return str(num) + hashlib.shake_256(h).hexdigest(vars.VERGEN_HASH_SIZE)
    else:
        if vars.PRINT_CUPDEFS_ERROR:
            utils.perror('Генератор номера не инициализирован')



import colorama
from CupDefs import utils
from CupDefs import vars
from CupDefs import vergen


def init(*args: str) -> bool:
    use_color = True

    if 'no_color' in args:
        utils.pwarn = utils.pdone = utils.perror = utils.pinfo = print
        use_color = False
    else:
        colorama.init(autoreset=True)
    if 'no_vergen' not in args:
        vergen.init_vergen()
    if 'no_logo' not in args:
        utils.print_logo(use_color)
    return True



import colorama
import os
from CupDefs import vars


def perror(text: str):
    print(colorama.Fore.RED + text)


def pwarn(text: str):
    print(colorama.Fore.YELLOW + text)


def pdone(text: str):
    print(colorama.Fore.GREEN + text)


def pinfo(text: str):
    print(colorama.Fore.CYAN + text)


def print_logo(colorize: bool):
    with open(os.path.join(vars.__PACKAGE_DIR, 'logo'), 'r') as logo:
        logo_text = []
        for line in logo.readlines():
            logo_text.append(line.rstrip('\n'))
        for line in logo_text:
            for s in line:
                if s == 'A':
                    if colorize:
                        print(colorama.Back.YELLOW + ' ', end=colorama.Style.RESET_ALL)
                    else:
                        print('#', end='')
                else:
                    print(' ', end='')
            print()
